@@ -48,10 +48,11 @@ typedef enum {
 	PED_PARTITION_LVM=6,
 	PED_PARTITION_LBA=7,
 	PED_PARTITION_HPSERVICE=8,
-	PED_PARTITION_PALO=9
+	PED_PARTITION_PALO=9,
+	PED_PARTITION_PREP=10
 } PedPartitionFlag;
 #define PED_PARTITION_FIRST_FLAG	PED_PARTITION_BOOT
-#define PED_PARTITION_LAST_FLAG		PED_PARTITION_PALO
+#define PED_PARTITION_LAST_FLAG		PED_PARTITION_PREP
 
 typedef enum {
 	PED_DISK_TYPE_EXTENDED=1,	/* supports extended partitions */
