@@ -53,13 +53,13 @@ static const char * _amiga_get_bstr (char * bstr) {
 	return cstr;
 }
 
-#define	IDNAME_RIGIDDISK	0x5244534B	/* 'RDSK' */
-#define IDNAME_BADBLOCK		0x42414442	/* 'BADB' */
-#define	IDNAME_PARTITION	0x50415254	/* 'PART' */
-#define IDNAME_FILESYSHEADER	0x46534844	/* 'FSHD' */
-#define IDNAME_LOADSEG		0x4C534547	/* 'LSEG' */
-#define IDNAME_BOOT		0x424f4f54	/* 'BOOT' */
-#define IDNAME_FREE		0xffffffff	
+#define	IDNAME_RIGIDDISK	(uint32_t)0x5244534B	/* 'RDSK' */
+#define IDNAME_BADBLOCK		(uint32_t)0x42414442	/* 'BADB' */
+#define	IDNAME_PARTITION	(uint32_t)0x50415254	/* 'PART' */
+#define IDNAME_FILESYSHEADER	(uint32_t)0x46534844	/* 'FSHD' */
+#define IDNAME_LOADSEG		(uint32_t)0x4C534547	/* 'LSEG' */
+#define IDNAME_BOOT		(uint32_t)0x424f4f54	/* 'BOOT' */
+#define IDNAME_FREE		(uint32_t)0xffffffff	
 
 static const char *
 _amiga_block_id (uint32_t id) {
@@ -326,7 +326,7 @@ struct Linked2Block {
     uint32_t	lk2_Reverved[13];
     uint32_t	lk2_Linked;		/* Secondary linked list */
 };
-#define LINK_END	0xffffffff
+#define LINK_END	(uint32_t)0xffffffff
 #define LNK(pos)	((struct LinkedBlock *)(pos))
 #define LNK2(pos)	((struct Linked2Block *)(pos))
 
