@@ -32,6 +32,19 @@
 #  define _(String) (String)
 #endif /* ENABLE_NLS */
 
+/* Default size for volhdr part, same val as IRIX's fx uses */
+#define PTYPE_VOLHDR_DFLTSZ 4096
+
+/* Partition numbers that seem to be strongly held convention */
+#define PNUM_VOLHDR 8
+#define PNUM_VOLUME 10
+
+/* Other notes of interest:
+ *  PED_PARTITION_EXTENDED is used for volume headers
+ *  PED_PARTITION_LOGICAL  is used for bootfiles
+ *  PED_PARTITION_NORMAL   is used for all else
+ */
+
 typedef struct _DVHDiskData {
 	struct device_parameters	dev_params;
 	int				swap;	/* part num of swap, 0=none */
@@ -41,7 +54,7 @@ typedef struct _DVHDiskData {
 
 typedef struct _DVHPartData {
 	int	type;
-	char	name[9];		/* boot volumes only */
+	char	name[VDNAMESIZE + 1];	/* boot volumes only */
 	int	real_file_size;		/* boot volumes only */
 } DVHPartData;
 
@@ -93,11 +106,10 @@ dvh_alloc (PedDevice* dev)
 	dvh_disk_data->boot = 0;
 
 	volume_part = ped_partition_new (disk, PED_PARTITION_EXTENDED, NULL,
-					 1,
-					 PED_MIN (dev->length / 10,
-						  10 * 1024 * 1024 / 512));
+					 1, PTYPE_VOLHDR_DFLTSZ - 1);
 	if (!volume_part)
 		goto error_free_disk_specific;
+	volume_part->num = PNUM_VOLHDR + 1;
 	constraint_any = ped_constraint_any (dev);
 	if (!ped_disk_add_partition (disk, volume_part, constraint_any))
 		goto error_destroy_constraint_any;
@@ -195,11 +207,10 @@ _handle_no_volume_header (PedDisk* disk)
 			disk, PED_PARTITION_EXTENDED,
 			NULL,
 			first_free_space->geom.start,
-			first_free_space->geom.start - 1
-			   + PED_MIN(first_free_space->geom.length,
-				     1024 * 16 * (1024/512))); /* FIXME ? */
+			first_free_space->geom.start + PTYPE_VOLHDR_DFLTSZ - 1);
 		if (!part)
 			goto error;
+		part->num = PNUM_VOLHDR + 1;
 		constraint_exact = ped_constraint_exact (&part->geom);
 		if (!constraint_exact)
 			goto error_destroy_part;
@@ -341,6 +352,7 @@ dvh_read (PedDisk* disk)
 
 		if (!vh.vh_pt[i].pt_nblks)
 			continue;
+		/* Skip the whole-disk partition, parted disklikes overlap */
 		if (PED_BE32_TO_CPU (vh.vh_pt[i].pt_type) == PTYPE_VOLUME)
 			continue;
 
@@ -422,6 +434,7 @@ _generate_partition (PedPartition* part, struct partition_table* pt)
 {
 	DVHPartData*	dvh_part_data = part->disk_specific;
 
+	/* Assert not a bootfile */
 	PED_ASSERT ((part->type & PED_PARTITION_LOGICAL) == 0, return);
 
 	if (part->geom.start == 1) {
@@ -443,13 +456,14 @@ _generate_boot_file (PedPartition* part, struct volume_directory* vd)
 {
 	DVHPartData*	dvh_part_data = part->disk_specific;
 
+	/* Assert it's a bootfile */
 	PED_ASSERT ((part->type & PED_PARTITION_LOGICAL) != 0, return);
 
 	vd->vd_nbytes = PED_CPU_TO_BE32 (dvh_part_data->real_file_size);
 	vd->vd_lbn = PED_CPU_TO_BE32 (part->geom.start);
 
+	memset (vd->vd_name, 0, VDNAMESIZE);
 	strncpy (vd->vd_name, dvh_part_data->name, VDNAMESIZE);
-	vd->vd_name[VDNAMESIZE] = 0;
 }
 
 static int
@@ -476,6 +490,13 @@ dvh_write (PedDisk* disk)
 	}
 
 	vh.vh_dp = dvh_disk_data->dev_params;
+	/* Set up rudimentary device geometry */
+	vh.vh_dp.dp_cyls
+		= PED_CPU_TO_BE16 ((short)disk->dev->bios_geom.cylinders);
+	vh.vh_dp.dp_trks0 = PED_CPU_TO_BE16 ((short)disk->dev->bios_geom.heads);
+	vh.vh_dp.dp_secs
+		= PED_CPU_TO_BE16 ((short)disk->dev->bios_geom.sectors);
+	vh.vh_dp.dp_secbytes = PED_CPU_TO_BE16 ((short)disk->dev->sector_size);
 
 	for (i = 0; i < NPARTAB; i++) {
 		PedPartition* part = ped_disk_get_partition (disk, i + 1);
@@ -483,10 +504,14 @@ dvh_write (PedDisk* disk)
 			_generate_partition (part, &vh.vh_pt[i]);
 	}
 
-	/* whole disk partition */
-	vh.vh_pt[10].pt_nblks = PED_CPU_TO_BE32 (disk->dev->length);
-	vh.vh_pt[10].pt_firstlbn = PED_CPU_TO_BE32 (0);
-	vh.vh_pt[10].pt_type = PED_CPU_TO_BE32 (PTYPE_VOLUME);
+	/* whole disk partition
+	 * This is only ever written here, and never modified 
+	 * (or even shown) as it must contain the entire disk, 
+	 * and parted does not like overlapping partitions
+	 */
+	vh.vh_pt[PNUM_VOLUME].pt_nblks = PED_CPU_TO_BE32 (disk->dev->length);
+	vh.vh_pt[PNUM_VOLUME].pt_firstlbn = PED_CPU_TO_BE32 (0);
+	vh.vh_pt[PNUM_VOLUME].pt_type = PED_CPU_TO_BE32 (PTYPE_VOLUME);
 
 	for (i = 0; i < NVDIR; i++) {
 		PedPartition* part = ped_disk_get_partition (disk,
@@ -550,6 +575,7 @@ dvh_partition_duplicate (const PedPartition* part)
 				       part->geom.start, part->geom.end);
 	if (!result)
 		goto error;
+	result->num = part->num;
 
 	if (!ped_partition_is_active (part)) {
 		result->disk_specific = NULL;
@@ -594,6 +620,7 @@ dvh_partition_set_system (PedPartition* part, const PedFileSystemType* fs_type)
 		return 1;
 	}
 
+	/* Is this a bootfile? */
 	if (part->type == PED_PARTITION_LOGICAL)
 		return 1;
 
@@ -715,6 +742,7 @@ dvh_partition_set_name (PedPartition* part, const char* name)
 	DVHPartData* dvh_part_data = part->disk_specific;
 
 	if (part->type == PED_PARTITION_LOGICAL) {
+		/* Bootfile */
 		strncpy (dvh_part_data->name, name, VDNAMESIZE);
 		dvh_part_data->name[VDNAMESIZE] = 0;
 	} else {
@@ -741,7 +769,7 @@ _get_strict_primary_constraint (PedDisk* disk)
         PedAlignment    start_align;
         PedAlignment    end_align;
         PedGeometry     max_geom;
-	PedSector	block = dev->sectors * dev->heads;
+	PedSector	block = dev->bios_geom.sectors * dev->bios_geom.heads;
 	PedSector	start_lower_bound;
 	PedPartition*	extended_partition = ped_disk_extended_partition (disk);
 
@@ -811,6 +839,7 @@ dvh_partition_enumerate (PedPartition* part)
 	_flush_stale_flags (part->disk);
 
 	if (part->type & PED_PARTITION_LOGICAL) {
+		/* Bootfile */
 		for (i = 1 + NPARTAB; i <= NPARTAB + NVDIR; i++) {
 			if (!ped_disk_get_partition (part->disk, i)) {
 				part->num = i;
@@ -818,10 +847,13 @@ dvh_partition_enumerate (PedPartition* part)
 			}
 		}
 		PED_ASSERT (0, return 0);
+	} else if (part->type & PED_PARTITION_EXTENDED) {
+		/* Volheader */
+		part->num = PNUM_VOLHDR + 1;
 	} else {
 		for (i = 1; i <= NPARTAB; i++) {
 			/* reserved for full volume partition */
-			if (i == 11)
+			if (i == PNUM_VOLUME + 1)
 				continue;
 
 			if (!ped_disk_get_partition (part->disk, i)) {
