@@ -23,8 +23,8 @@
 #include <parted/debug.h>
 #include <string.h>
 
-#include <libintl.h>
 #if ENABLE_NLS
+#  include <libintl.h>
 #  define _(String) dgettext (PACKAGE, String)
 #else
 #  define _(String) (String)
@@ -239,7 +239,8 @@ ped_file_system_clobber (PedGeometry* geom)
 	while ((fs_type = ped_file_system_type_get_next (fs_type))) {
 		PedGeometry*	probed;
 
-		PED_ASSERT (fs_type->ops->clobber != NULL, return 0);
+		if (!fs_type->ops->clobber)
+			continue;
 
 		probed = ped_file_system_probe_specific (fs_type, geom);
 		if (!probed) {
