@@ -355,8 +355,8 @@ _inode_update_size(struct ext2_fs *fs, struct ext2_inode *inode, int delta)
 	inode->i_blocks = PED_CPU_TO_LE32(EXT2_INODE_BLOCKS(*inode)
 		       			  + delta * i512perblock);
 	size = EXT2_INODE_SIZE(*inode) + delta * fs->blocksize;
-	inode->i_size = PED_CPU_TO_LE32(size % 0x100000000);
-	inode->i_size_high = PED_CPU_TO_LE32(size / 0x100000000);
+	inode->i_size = PED_CPU_TO_LE32(size % (1LL << 32));
+	inode->i_size_high = PED_CPU_TO_LE32(size / (1LL << 32));
 	inode->i_mtime = PED_CPU_TO_LE32(time(NULL));
 }
 
@@ -748,17 +748,19 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, int state)
 		& EXT3_FEATURE_INCOMPAT_RECOVER))
 	{
 		if (ped_exception_throw (
-			PED_EXCEPTION_WARNING, PED_EXCEPTION_IGNORE_CANCEL,
+			PED_EXCEPTION_ERROR, PED_EXCEPTION_IGNORE_CANCEL,
 			_("Filesystem was not cleanly unmounted!  "
-			  "You should e2fsck."))
-				== PED_EXCEPTION_CANCEL)
+			  "You should e2fsck.  Modifying an unclean filesystem "
+			  "could cause severe corruption."))
+				!= PED_EXCEPTION_IGNORE)
 			goto error_free_fs;
 	}
 
 	fs->dynamic_version = EXT2_SUPER_REV_LEVEL (fs->sb) > 0;
 
 	if ((EXT2_SUPER_FEATURE_COMPAT(fs->sb)
-	                & ~EXT3_FEATURE_COMPAT_HAS_JOURNAL) ||
+	                & ~(EXT3_FEATURE_COMPAT_HAS_JOURNAL |
+			    EXT2_FEATURE_COMPAT_HAS_DIR_INDEX)) ||
 	    (EXT2_SUPER_FEATURE_INCOMPAT(fs->sb)
 	    		& ~(EXT2_FEATURE_INCOMPAT_FILETYPE |
 			    EXT3_FEATURE_INCOMPAT_RECOVER)) ||
