@@ -17,11 +17,9 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-static const char _tune_c[] = "$Id: tune.c,v 1.2 1999/09/02 12:07:14 buytenh Exp $";
-
 #include "config.h"
 
-
+#ifndef DISCOVER_ONLY
 
 /*
  * maybe i'll make this all command-line configurable one day
@@ -39,3 +37,4 @@ int ext2_hash_bits = 8;
    as you can. The smaller this is, the more disk I/O is required for
    doing relocations. */
 int ext2_relocator_pool_size = 4096;
+#endif /* !DISCOVER_ONLY */
