@@ -32,6 +32,31 @@ fat_min_cluster_size (FatType fat_type) {
 	return 0;
 }
 
+static PedSector
+_smallest_power2_over (PedSector ceiling)
+{
+	PedSector	result = 1;
+
+	while (result < ceiling)
+		result *= 2;
+
+	return result;
+}
+
+/* returns the minimum size of clusters for a given filesystem type */
+PedSector
+fat_recommend_min_cluster_size (FatType fat_type, PedSector size) {
+	switch (fat_type) {
+		case FAT_TYPE_FAT12: return 1;
+		case FAT_TYPE_FAT16: return fat_min_cluster_size(fat_type);
+		case FAT_TYPE_FAT32:
+			return PED_MAX(_smallest_power2_over(size
+						/ MAX_FAT32_CLUSTERS),
+				       fat_min_cluster_size (fat_type));
+	}
+	return 0;
+}
+
 /* returns the maxmimum size of clusters for a given filesystem type */
 PedSector
 fat_max_cluster_size (FatType fat_type) {
@@ -208,7 +233,7 @@ fat_calc_sizes (PedSector size, PedSector align, FatType fat_type,
 	PED_ASSERT (out_cluster_count != NULL, return 0);
 	PED_ASSERT (out_fat_size != NULL, return 0);
 
-	for (cluster_sectors = fat_min_cluster_size (fat_type);
+	for (cluster_sectors = fat_recommend_min_cluster_size (fat_type, size);
 	     cluster_sectors <= fat_max_cluster_size (fat_type);
 	     cluster_sectors *= 2) {
 		if (calc_sizes (size, align, fat_type, root_dir_sectors,
@@ -219,6 +244,17 @@ fat_calc_sizes (PedSector size, PedSector align, FatType fat_type,
 		}
 	}
 
+	for (cluster_sectors = fat_recommend_min_cluster_size (fat_type, size);
+	     cluster_sectors >= fat_min_cluster_size (fat_type);
+	     cluster_sectors /= 2) {
+		if (calc_sizes (size, align, fat_type, root_dir_sectors,
+				cluster_sectors,
+			        out_cluster_count, out_fat_size)) {
+			*out_cluster_sectors = cluster_sectors;
+			return 1;
+		}
+	}
+
 	/* only make the cluster size really small (<4k) if a bigger one is
 	 * isn't possible.  Windows never makes FS's like this, but it
 	 * seems to work...  (do more tests!)
