@@ -23,7 +23,7 @@
 #include "context.h"
 
 /* the big important one :-) */
-extern int fat_duplicate_clusters (FatOpContext* ctx);
+extern int fat_duplicate_clusters (FatOpContext* ctx, PedTimer* timer);
 
 #endif /* PED_FAT_CLSTDUP_H_INCLUDED */
 
