@@ -183,12 +183,14 @@ ped_geometry_read (const PedGeometry* geom, void* buffer, PedSector start,
 		   PedSector count)
 {
 	int		exception_status;
-	PedSector	real_start = geom->start + start;
+	PedSector	real_start;
 
 	PED_ASSERT (geom != NULL, return 0);
 	PED_ASSERT (buffer != NULL, return 0);
 	PED_ASSERT (start >= 0, return 0);
 	PED_ASSERT (count >= 0, return 0);
+	
+	real_start = geom->start + start;
 
 	if (real_start + count - 1 > geom->end) {
 		exception_status = ped_exception_throw (
@@ -217,12 +219,14 @@ ped_geometry_write (PedGeometry* geom, const void* buffer, PedSector start,
 		    PedSector count)
 {
 	int		exception_status;
-	PedSector	real_start = geom->start + start;
+	PedSector	real_start;
 
 	PED_ASSERT (geom != NULL, return 0);
 	PED_ASSERT (buffer != NULL, return 0);
 	PED_ASSERT (start >= 0, return 0);
 	PED_ASSERT (count >= 0, return 0);
+	
+	real_start = geom->start + start;
 
 	if (real_start + count - 1 > geom->end) {
 		exception_status = ped_exception_throw (
