@@ -224,7 +224,8 @@ gnu_new (const char* path)
 {
 	PedDevice*	dev;
 	GNUSpecific*	arch_specific;
-	error_t err;
+	error_t         ro_err, rw_err;
+	int             ispath;
 
 	PED_ASSERT (path != NULL, return NULL);
 
@@ -236,18 +237,32 @@ gnu_new (const char* path)
 	arch_specific->consume = 1;
 
  retry_open:
-	err = store_typed_open (dev->path, 0, NULL, &arch_specific->store);
-	if (err) {
-		error_t rw_err = err;
+	/* Try read-write. */
+	if (strchr (dev->path, '/') != NULL) {
+	        /* We set this to prevent having to use strchr more then once. */
+	        ispath = 1; 
 
-		err = store_typed_open (dev->path, STORE_READONLY, NULL,
-					&arch_specific->store);
-		if (err) {
-			if (ped_exception_throw (
-				PED_EXCEPTION_ERROR,
+	        rw_err = store_open (dev->path, 0, NULL, &arch_specific->store);
+	} else { 
+	        rw_err = store_typed_open (dev->path, 0, NULL, &arch_specific->store);
+	}        
+
+	/* Try readonly. */
+	if (rw_err) {
+	  if (ispath) {
+	        ro_err = store_open (dev->path, STORE_READONLY, NULL,
+				 &arch_specific->store);
+	  } else {
+	        ro_err = store_typed_open (dev->path, STORE_READONLY, NULL,
+				       &arch_specific->store);
+	  }
+                 
+	if (ro_err) {
+	        if (ped_exception_throw (
+			        PED_EXCEPTION_ERROR,
 				PED_EXCEPTION_RETRY_CANCEL,
 				_("Error opening %s: %s"),
-				dev->path, strerror (err))
+				dev->path, strerror (ro_err))
 					!= PED_EXCEPTION_RETRY) {
 				return NULL;
 			} else
