@@ -307,7 +307,7 @@ _device_probe_type (PedDevice* dev)
 	} else if (_is_cpqarray_major (dev_major) && (dev_minor % 0x10 == 0)) {
 		dev->type = PED_DEVICE_CPQARRAY;
 	} else {
-		dev->type = PED_DEVICE_FILE;
+		dev->type = PED_DEVICE_UNKNOWN;
 	}
 
 	return 1;
@@ -1434,7 +1434,8 @@ linux_probe_all ()
 static char*
 _device_get_part_path (PedDevice* dev, int num)
 {
-	int		result_len = strlen (dev->path) + 16;
+	int		path_len = strlen (dev->path);
+	int		result_len = path_len + 16;
 	char*		result;
 
 	result = (char*) ped_malloc (result_len);
@@ -1442,20 +1443,14 @@ _device_get_part_path (PedDevice* dev, int num)
 		return NULL;
 
 	/* FIXME: how do we know if a device is devfs or not? */
-	if (_have_devfs()) {
-		int	len = strlen (dev->path);
-
+	if (_have_devfs() && !strcmp (dev->path + path_len - 5, "/disc")) {
+		/* replace /disc with /path%d */
 		strcpy (result, dev->path);
-
-		/* strip "/disc" from the end */
-		if (!strcmp (result + len - 5, "/disc"))
-			len -= 5;
-
-	        snprintf (result + len, 16, "/part%d", num);
+	        snprintf (result + path_len - 5, 16, "/part%d", num);
 	} else if (dev->type == PED_DEVICE_DAC960
 			|| dev->type == PED_DEVICE_CPQARRAY
 			|| dev->type == PED_DEVICE_ATARAID
-			|| isdigit (dev->path[strlen(dev->path) - 1]))
+			|| isdigit (dev->path[path_len - 1]))
 	        snprintf (result, result_len, "%sp%d", dev->path, num);
 	else
 	        snprintf (result, result_len, "%s%d", dev->path, num);
@@ -1469,6 +1464,19 @@ linux_partition_get_path (const PedPartition* part)
 	return _device_get_part_path (part->disk->dev, part->num);
 }
 
+static dev_t
+_partition_get_part_dev (const PedPartition* part)
+{
+	struct stat dev_stat;
+	int dev_major, dev_minor;
+
+	if (!_device_stat (part->disk->dev, &dev_stat))
+		return 0;
+	dev_major = major (dev_stat.st_rdev);
+	dev_minor = minor (dev_stat.st_rdev);
+	return makedev (dev_major, dev_minor + part->num);
+}
+
 static int
 _mount_table_search (const char* file_name, dev_t dev)
 {
@@ -1493,6 +1501,14 @@ _mount_table_search (const char* file_name, dev_t dev)
 	return 0;
 }
 
+static int
+_partition_is_mounted_by_dev (dev_t dev)
+{
+	return  _mount_table_search( "/proc/mounts", dev)
+		|| _mount_table_search( "/proc/swaps", dev)
+		|| _mount_table_search( "/etc/mtab", dev);
+}
+
 static int
 _partition_is_mounted_by_path (const char *path)
 {
@@ -1501,25 +1517,17 @@ _partition_is_mounted_by_path (const char *path)
 		return 0;
 	if (!S_ISBLK(part_stat.st_mode))
 		return 0;
-	return  _mount_table_search( "/proc/mounts", part_stat.st_rdev)
-		|| _mount_table_search( "/proc/swaps", part_stat.st_rdev)
-		|| _mount_table_search( "/etc/mtab", part_stat.st_rdev);
+	return _partition_is_mounted_by_dev (part_stat.st_rdev);
 }
 
 static int
 _partition_is_mounted (const PedPartition *part)
 {
-	char* part_name;
-	int result;
-
+	dev_t dev;
 	if (!ped_partition_is_active (part))
 		return 0;
-	part_name = _device_get_part_path (part->geom.dev, part->num);
-	if (!part_name)
-		return 1;
-	result = _partition_is_mounted_by_path (part_name);
-	ped_free (part_name);
-	return result;
+	dev = _partition_get_part_dev (part);
+	return _partition_is_mounted_by_dev (dev);
 }
 
 static int
@@ -1644,14 +1652,8 @@ _disk_sync_part_table (PedDisk* disk)
 					continue;
 
 			/* add the (possibly modified or new) partition */
-			if (!_blkpg_add_partition (disk, part)) {
-				ped_exception_throw (
-					PED_EXCEPTION_ERROR,
-					PED_EXCEPTION_IGNORE,
-					"Error adding partition %d: %s",
-					i, strerror(errno));
+			if (!_blkpg_add_partition (disk, part))
 				ret = 0;
-			}
 		}
 	}
 
@@ -1708,10 +1710,11 @@ linux_disk_commit (PedDisk* disk)
 		 * created.  Therefore, if using DevFS, we must get the kernel
 		 * to re-read and grok the partition table.
 		 */
-		if (_have_blkpg () && !_have_devfs ())
-			return _disk_sync_part_table (disk);
-		else
-			return _kernel_reread_part_table (disk->dev);
+		if (_have_blkpg () && !_have_devfs ()) {
+			if (_disk_sync_part_table (disk))
+				return 1;
+		}
+		return _kernel_reread_part_table (disk->dev);
 	}
 	return 1;
 }
