@@ -138,22 +138,17 @@ _partition_warn_busy (PedPartition* part)
 	char* path = ped_partition_get_path (part);
 
 	if (ped_partition_is_busy (part)) {
-		if (ped_exception_throw (
+		ped_exception_throw (
 			PED_EXCEPTION_ERROR,
-			PED_EXCEPTION_IGNORE_CANCEL,
-			_("Partition %s is being used.  Modifying it while it "
-			  "is in use could cause severe corruption."),
-			path)
-				!= PED_EXCEPTION_IGNORE)
-			goto error_free_path;
+			PED_EXCEPTION_CANCEL,
+			_("Partition %s is being used.  You must unmount it "
+			  "before you modify it with Parted."),
+			path);
+		ped_free (path);
+		return 0;
 	}
 	ped_free (path);
 	return 1;
-
-error_free_path:
-	ped_free (path);
-error:
-	return 0;
 }
 
 static int
@@ -889,8 +884,8 @@ do_print (PedDevice** dev)
 		printf ("%-5d ", part->num);
 
 		printf ("%10.3f %10.3f  ",
-			(int) part->geom.start * 1.0 / MEGABYTE_SECTORS,
-			(int) part->geom.end * 1.0 / MEGABYTE_SECTORS);
+			part->geom.start * 1.0 / MEGABYTE_SECTORS,
+			part->geom.end * 1.0 / MEGABYTE_SECTORS);
 
 		if (has_extended)
 			printf ("%-9s ",
@@ -967,8 +962,8 @@ _rescue_add_partition (PedPartition* part)
 		  "Do you want to add it to the partition table?"),
 		fs_type->name,
 		ped_partition_type_get_name (part->type),
-		(int) probed->start * 1.0 / MEGABYTE_SECTORS,
-		(int) probed->end * 1.0 / MEGABYTE_SECTORS);
+		probed->start * 1.0 / MEGABYTE_SECTORS,
+		probed->end * 1.0 / MEGABYTE_SECTORS);
 
 	switch (ex_opt) {
 		case PED_EXCEPTION_CANCEL: return -1;
